var util   = require('util'),
    sinon  = require('sinon'),
    should = require('should'),
    http   = require('http'),
    needle = require('./../'),
		EventEmitter = require('events').EventEmitter;

var FakeRequest = function(args, callback){
	this.end = function(){ /* noop */ };
	if (callback) callback();
}

util.inherits(FakeRequest, EventEmitter);

describe('Needle', function(){
	
	var req, spy;
	
	var disableRequests = function(){
	  spy = sinon.stub(http, 'request');
		// spyOn(http, 'request').andReturn(new FakeRequest());		
		// spyOn(this, 'setTimeout').andReturn(true); // dont wait for default timeout
	}
	
	describe('module', function(){
		
	  it('should expose version', function(){ 
	    should.exist(needle.version);
      needle.version.should.be.a('string');
		});

		it('should export head/get/post/put/delete methods', function(){
			Object.keys(needle).join(' ').should.equal('version head get post put delete');
		})

	})

	describe('requests', function(){
		
		// common for all methods

		describe('by default', function(){
		  
		  describe('URLs', function(){
		    
		    var url = require('url'); 
		    
		    afterEach(function(){
		      spy.restore();
		    })

  			it('should add http:// to request URL if missing', function(){
  			  spy = sinon.spy(needle, 'send_request');
  				needle.get('www.google.com');
  				spy.args.should.equal('http://www.google.com');
  				spy.restore();
  			})

  			it('should not verify if host is valid', function(){
  			  spy = sinon.spy(needle, 'send_request');
  			  needle.get('x1x1x1');
  			  console.log(spy);
  				spy.args.should.equal('http://x1x1x1');
  			})

		  })
		  
		  describe('headers', function(){

  			it('should set Host, Connection, User-Agent and Accept headers', function(){
  				var req = needle.get('some.com/url');
  				Object.keys(req._headers).sort().join(' ').should.equal('accept connection host user-agent');
  			})

  			it('should not ask for a deflated response', function(){
  			  // spy = sinon.spy(needle, 'send_request');
  				needle.get('google.com');
  				// spy.calledWith.should.not.include( headers['Accept-Encoding'] );
  				// spy.restore();
  			})

		  })
		  
		  describe('other options', function(){

        /* TODO
        			it('should set a 10 sec timeout timer', function(){
        				spyOn(this, 'setTimeout');
        				needle.get('server.com/with/some/url')
        				expect(setTimeout).toHaveBeenCalledWith(function(){}, 10000);
        			})
        */

		  })
			
		})
		
		describe('when proxied', function(){
			
			beforeEach(function(){
				req = needle.post('finalhost.com/endpoint', { proxy: 'http://localhost:5141' });
			})
			
			it('should set Host header to final host', function(){
				req._headers['host'].should.equal('finalhost.com');
			})
			
			it('should set the original (full) URL as the request path', function(){
				req.path.should.equal('/endpoint');
			})
			
		})
		
		// common stuff for all requests
		
		describe('headers', function(){
			
			it('should accept setting custom headers', function(){
				var req = needle.get('random.url', { headers: {'X-Custom-Header': '12345-something'} });
				req._headers['x-custom-header'].should.equal('12345-something');
			})
			
			describe('HTTP basic auth', function(){
				
				it('should set an Authorization header when username & password are present', function(){

				})

				it('should not set Authorization header if only username or password are set', function(){

				})
				
				it('Authorization header should be Base64 encoded (& decodeable)', function(){
					
				})
				
			})

		})

		// specific for each method
		
		describe('GET', function(){
			
			it('should work', function(){
				
			})
			
			it('should not send any data')
			
		})
		
		describe('POST', function(){

			it('should require data', function(){
				
			})

			it('should work', function(){
				
			})
			
			it('should default to non-multipart', function(){
				
			})
			
			it('should add a Content-Length header', function(){
				
			})
			
			it('should send the data in ut8 encoding', function(){
				
			})
			
			describe('with multipart', function(){
				
				it('should also have a Content-Length header', function(){
					
				})
				
				it('should send the data in binary encoding', function(){
					
				})
				
			})
			
		})
		
		describe('PUT', function(){

			it('should work', function(){
				
			})

			it('should require data', function(){
				
			})

		})
		
		describe('DELETE', function(){

			it('should work', function(){
				
			})

		})
		
	})
	
	describe('errors', function(){
		

		describe('invalid URL', function(){
			

			describe('should not raise exceptions', function(){

			})

			describe('callback should include an error', function(){

			})

			describe('callback should not include a response', function(){

			})

			describe('callback should not include a body', function(){

			})
			
		})
		
		describe('non-existant host', function(){
			
			it('should not work', function(){
				
			})

			describe('should not raise exceptions', function(){

			})

			describe('callback should include an error', function(){

			})

			describe('callback should not include a response', function(){

			})

			describe('callback should not include a body', function(){

			})

		})
		
		describe('timeout', function(){
			
			it('should abort the request', function(){
				
			})
			
			describe('should not raise exceptions', function(){

			})

			describe('callback should include an error', function(){

			})

			describe('callback should not include a response', function(){

			})

			describe('callback should not include a body', function(){

			})			
		})
		
	})

});